# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"logger": "95_cli.ipynb",
         "CocoElement": "01_coco.ipynb",
         "CocoInfo": "01_coco.ipynb",
         "CocoLicense": "01_coco.ipynb",
         "CocoImage": "01_coco.ipynb",
         "CocoAnnotation": "01_coco.ipynb",
         "CocoObjectDetectionAnnotation": "01_coco.ipynb",
         "CocoCategory": "01_coco.ipynb",
         "CocoObjectDetectionCategory": "01_coco.ipynb",
         "CocoDataset": "01_coco.ipynb",
         "CocoObjectDetectionDataset": "01_coco.ipynb",
         "get_dataset_class": "01_coco.ipynb",
         "MAP_COCO_TYPE_TO_DATASET_CLASS": "01_coco.ipynb",
         "merge_datasets": "01_coco.ipynb",
         "shuffle": "01_coco.ipynb",
         "cut_annotations_per_category": "01_coco.ipynb",
         "remove_invalid_elements": "01_coco.ipynb",
         "load_json_file": "02_json_file.ipynb",
         "dump_json_file": "02_json_file.ipynb",
         "load_json_tree": "03_json_tree.ipynb",
         "dump_json_tree": "03_json_tree.ipynb",
         "load_crop_tree": "04_crop_tree.ipynb",
         "dump_crop_tree": "04_crop_tree.ipynb",
         "TimerMutable": "90_utils.ipynb",
         "measure_time": "90_utils.ipynb",
         "log_elapsed_time": "90_utils.ipynb",
         "sort_dict": "90_utils.ipynb",
         "sanitize_filename": "90_utils.ipynb",
         "read_image": "90_utils.ipynb",
         "download_image": "90_utils.ipynb",
         "cut_bbox": "90_utils.ipynb",
         "write_image": "90_utils.ipynb",
         "get_parser": "95_cli.ipynb",
         "main": "95_cli.ipynb"}

modules = ["coco.py",
           "json_file.py",
           "json_tree.py",
           "crop_tree.py",
           "utils.py",
           "cli.py"]

doc_url = "https://neuro-inc.github.io/cocorepr/cocorepr/"

git_url = "https://github.com/neuro-inc/cocorepr"

def custom_doc_links(name): return None
